//-----------------------------------------------------------------------------
// Title      : bsp_ub048_mk3_counter1
//-----------------------------------------------------------------------------
// Funktion   : Counter Test1
// Schaltung  : myAVR MK3
//-----------------------------------------------------------------------------
// Prozessor  : ATMega2560
// Takt       : 16 MHz
// Sprache    : C
// Date       : 16.06.2012
// Version    : 1.0
// Autor      : UB
//-----------------------------------------------------------------------------
#define F_CPU 16000000
#include <avr/io.h>
#include <ub_mk3_2560/MK3_2560_COUNTER.h>
#include <ub_mk3_2560/MK3_2560_LED.h>
#include <ub_mk3_2560/MK3_2560_KEY.h>
#include <ub_mk3_2560/MK3_2560_LCD.h>

//-----------------------------------------------------------------------------
// Hinweis : Quick-Jumper Port-L muss gesetzt sein
// - Quick-Jumper Port-K muss gesetzt sein
// - Quick-Jumper Port-C+A muss gesetzt sein
// - ein Taster muss an Port-D4 (ICP-Pin) angeschlossen werden
// - benutzt Counter Funktion um die Zeit
//   bis zu einem Tastendruck zu stoppen
// - zum neustart -> Reset druecken
//-----------------------------------------------------------------------------
int main (void)
{
	unsigned char wert=0;
	unsigned int n=0;

	MK3_INIT_COUNTER();				// init vom Counter
	MK3_INIT_LED();                 // Init der LEDs
	MK3_INIT_KEY();                 // Init der Buttons
	MK3_INIT_LCD();                 // Init vom Display

	MK3_LCD_CLEAR();
	MK3_LCD_LIGHT(true);

	MK3_LCD_STRING_AT(0,0,"Counter-Test 1");

	MK3_LCD_STRING_AT(0,16,"Button-1 = start");

	// 0x55 an den LEDs ausgeben
	MK3_LED_ZAHL(0x55);

	// warten bis Button-1 gedrueckt wurde
	while(MK3_KEY_PRESS(BUTTON1)==false) {
		// warten
	}

	MK3_LCD_STRING_AT(0,16,"ICP-1 = stop    ");

	// Counter sofort starten im Mode : Millisekunden
	// und Stop-Bedingung : ICP-Lo_Flanke
	MK3_COUNTER_SET(TYP_MS_1000,START_NOW,STOP_LO_ICP);

	while (true)
	{
		n++;
		if(n>1000) 
		{
			// kleine Pause
			n=0;
			wert++;
		}
		MK3_LED_ZAHL(wert);

		if(MK3_COUNTER_CHECK()==true) 
		{
			// wenn ein Event eingetreten ist
			MK3_LCD_STRING_AT(0,16,"Event eingetreten");

            // Zeit auslesen
			n=MK3_COUNTER_READ_TIME();
			MK3_LCD_UINT_AT(0,32,n);

			MK3_LCD_STRING_AT(35,32,"ms");
			
			// Event auslesen
			if(MK3_COUNTER_READ_EVENT()==ICP_EVENT) {
			  MK3_LCD_STRING_AT(0,48,"ICP-Event");
			}
			else if(MK3_COUNTER_READ_EVENT()==OVF_EVENT) {
			  MK3_LCD_STRING_AT(0,48,"Ueberlauf-Event");
			}

			while(1) {
				// endlosloop
			}
		}

	}
	return 0;
}
//-----------------------------------------------------------------------------
