//-----------------------------------------------------------------------------
// MK3_2560_COUNTER
// Version 1.0
// 18.06.2012 / UB
// "www.Mikrocontroller-4U.de"
// mc-4u(@)t-online.de
//-----------------------------------------------------------------------------
// Fuer das myAVR MK3 Board (oder MK2 Board)
// mit ATMega-2560 @ 16MHz (oder ATMega8)
// Counter Funktionen
//-----------------------------------------------------------------------------
// das File muss im Ordner
// "myAVR/WinAVR/AVR/Include/UB_MK3_2560/"
// liegen und mit #include <UB_MK3_2560/MK3_2560_COUNTER.h>
// eingebunden werden
//-----------------------------------------------------------------------------
// Belegt : @MK3 (ATMega2560)
//            Timer-1 (16bit)
//            eventuell Port-D Bit-4 (als Trigger Input [ICP1])
// Belegt : @MK2 (ATMega8)
//            Timer-1 (16bit)
//            eventuell Port-B Bit-0 (als Trigger Input [ICP1])
//
// Hinweis :
//
// > die DEFINE-Schalter "COUNTER_HW_MODE" und "COUNTER_QUARZ_TYP"
//   muessen zur Hardware passen !!
//-----------------------------------------------------------------------------


#include <avr/interrupt.h>

//-----------------------------------------------------------------------------
// Hier koennen Funktionen die nicht benutzt werden
// abgeschaltet werden...einfach den DEFINE von "1" auf "0" aendern
//-----------------------------------------------------------------------------




//-----------------------------------------------------------------------------
// Hardware auswahl
//-----------------------------------------------------------------------------
#define COUNTER_HW_MODE    0       // 0=MK3-Board, 1=MK1/MK2-Board/mySmartControl
#define COUNTER_QUARZ_TYP  0       // 0=16MHz, 1=8MHz, 2=3.6864MHz


//-----------------------------------------------------------------------------
// INFOS zur Benutzung :
// 01. Funktion "MK3_INIT_COUNTER" starten
// 02. Funktion "MK3_COUNTER_SET" aufrufen
// 03a. eventuell Counter manuell per "MK3_COUNTER_START" starten
// 03b. eventuell Counter manuell per "MK3_COUNTER_STOP" stoppen
// 04. warten bis "MK3_COUNTER_CHECK" true wird
// 05. Event auslesen per "MK3_COUNTER_READ_EVENT"
// 06. Zeit auslesen per "MK3_COUNTER_READ_TIME"
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Enumerationen fuer die Funktioen
//-----------------------------------------------------------------------------

typedef enum {
  TYP_US_1000 =0,  // Mode = mikro sekunden, maxwert=1000us
  TYP_US_10000,    // Mode = mikro sekunden, maxwert=10000us
  TYP_US_25000,    // Mode = mikro sekunden, maxwert=25000us
  TYP_MS_1000,     // Mode = milli sekunden, maxwert=1000ms
  TYP_MS_4000,     // Mode = milli sekunden, maxwert=4000ms
  TYP_S_4          // Mode = sekunden,       maxwert=4s
}CNT_TYP;

typedef enum {
  START_FKT =0,    // Start per Start-Funktion
  START_NOW,       // Start sofort nach der SET-Funktion
  START_LO_ICP,    // Start bei fallender Flanke am ICP-Pin
  START_HI_ICP     // Start bei steigender Flanke am ICP-Pin
}START_TRG;

typedef enum {
  STOP_FKT =0,     // Stop per Stop-Funktion
  STOP_LO_ICP,     // Stop bei fallender Flanke am ICP-Pin
  STOP_HI_ICP,     // Stop bei steigender Flanke am ICP-Pin
  CYCLE_ICP        // Zyklische Messung vom Signal an ICP-Pin
}STOP_TRG;

typedef enum {
  NO_EVENT =0,      // Counter laeuft noch
  ICP_EVENT,        // ICP event
  OVF_EVENT,        // Counter ueberlauf event
  STOP_EVENT        // Software STOP event
}CNT_EVENT;




//-----------------------------------------------------------------------------
// Funktionen die vom MAIN benutzt werden koennen
//-----------------------------------------------------------------------------
void MK3_INIT_COUNTER(void);                              // zum initialisieren
void MK3_COUNTER_SET(CNT_TYP c_typ, START_TRG c_on, STOP_TRG c_off);  // zum setzen
void MK3_COUNTER_START(void);                             // zum starten
void MK3_COUNTER_STOP(void);                              // zum stoppen
bool MK3_COUNTER_CHECK(void);                             // test auf Event
CNT_EVENT MK3_COUNTER_READ_EVENT(void);                   // Event auslesen
unsigned int MK3_COUNTER_READ_TIME(void);                 // Wert auslesen



//-----------------------------------------------------------------------------
// Port-Pin definition
//-----------------------------------------------------------------------------
#if COUNTER_HW_MODE==0
  // Hardware = myAVR MK3-Board
  #define MK3_DDR_ICP  DDRD    // DDR-Register vom ICP-Pin
  #define MK3_PORT_ICP PORTD   // Port vom ICP-Pin
  #define MK3_ICP_PIN  0x10    // Bit vom ICP-Pin
#else
  // Hardware = myAVR MK1/MK2-Board + mySmartControl
  #define MK3_DDR_ICP  DDRB    // DDR-Register vom ICP-Pin
  #define MK3_PORT_ICP PORTB   // Port vom ICP-Pin
  #define MK3_ICP_PIN  0x01    // Bit vom ICP-Pin
#endif




//-----------------------------------------------------------------------------
// interne defines
// m0=US_1000, m1=US_10000, m2=US_25000, 
// m3=MS_1000, m4=MS_4000,  m5=S_4
//-----------------------------------------------------------------------------
#if COUNTER_QUARZ_TYP==0
  // 16MHZ
  #define M0_CLK 0x02   // vorteiler=8
  #define M0_OVF 2000   // max=1ms
  #define M0_FKT 2.0
  #define M1_CLK 0x02   // vorteiler=8
  #define M1_OVF 20000  // max=10ms
  #define M1_FKT 2.0
  #define M2_CLK 0x02   // vorteiler=8
  #define M2_OVF 50000  // max=25ms
  #define M2_FKT 2.0
  #define M3_CLK 0x05   // vorteiler=1024
  #define M3_OVF 15625  // max=1s
  #define M3_FKT 15.625
  #define M4_CLK 0x05   // vorteiler=1024
  #define M4_OVF 62500  // max=4s
  #define M4_FKT 15.625
  #define M5_CLK 0x05   // vorteiler=1024
  #define M5_OVF 62500  // max=4s
  #define M5_FKT 15625.0
#elif COUNTER_QUARZ_TYP==1
  // 8MHz
  #define M0_CLK 0x02   // vorteiler=8
  #define M0_OVF 1000   // max=1ms
  #define M0_FKT 1.0
  #define M1_CLK 0x02   // vorteiler=8
  #define M1_OVF 10000  // max=10ms
  #define M1_FKT 1.0
  #define M2_CLK 0x02   // vorteiler=8
  #define M2_OVF 25000  // max=25ms
  #define M2_FKT 1.0
  #define M3_CLK 0x04   // vorteiler=256
  #define M3_OVF 31250  // max=1s
  #define M3_FKT 31.25
  #define M4_CLK 0x05   // vorteiler=1024
  #define M4_OVF 31250  // max=4s
  #define M4_FKT 7.8125
  #define M5_CLK 0x05   // vorteiler=1024
  #define M5_OVF 31250  // max=4s
  #define M5_FKT 7812.5
#elif COUNTER_QUARZ_TYP==2
  #define M0_CLK 0x01   // vorteiler=1
  #define M0_OVF 3687   // max=1ms
  #define M0_FKT 3.6864
  #define M1_CLK 0x01   // vorteiler=1
  #define M1_OVF 36864  // max=10ms
  #define M1_FKT 3.6864
  #define M2_CLK 0x02   // vorteiler=8
  #define M2_OVF 11520  // max=25ms
  #define M2_FKT 0.4608
  #define M3_CLK 0x04   // vorteiler=256
  #define M3_OVF 14400  // max=1s
  #define M3_FKT 14.4
  #define M4_CLK 0x05   // vorteiler=1024
  #define M4_OVF 14400  // max=4s
  #define M4_FKT 3.6
  #define M5_CLK 0x05   // vorteiler=1024
  #define M5_OVF 14400  // max=4s
  #define M5_FKT 3600.0
#endif

//-----------------------------------------------------------------------------
// interne Enumerationen
//-----------------------------------------------------------------------------
typedef enum {
  ICP_DISABLE =0,   // kein ICP-Mode
  ICP_CYCLE,        // zyklische Messung der Periodendauer
  ICP_WAIT_START,   // warte auf Start event
  ICP_WAIT_STOP     // warte auf Stop event
}ICP_MODE;

//-----------------------------------------------------------------------------
// interne Globale Variabeln
//-----------------------------------------------------------------------------
typedef struct {
  CNT_EVENT counter_event;  
  unsigned int counter_wert;
  double counter_faktor;
  unsigned char counter_clk;
  START_TRG start_trg;
  STOP_TRG stop_trg;  
  ICP_MODE akt_mode;
}Counter_Intern_t;
Counter_Intern_t COUNTER;


//-----------------------------------------------------------------------------
void MK3_INIT_COUNTER(void) {	// zum initialisieren

  // alle Variablen initialisieren

  COUNTER.counter_event=NO_EVENT;
  COUNTER.counter_wert=0;
  COUNTER.counter_faktor=1.0;
  COUNTER.counter_clk=0;
  COUNTER.start_trg=START_FKT;
  COUNTER.stop_trg=STOP_FKT;  
  COUNTER.akt_mode=ICP_DISABLE;

  SREG|=0x80; // Alle Interrupts einschalten
}

//-----------------------------------------------------------------------------
void MK3_COUNTER_SET(CNT_TYP c_typ, START_TRG c_on, STOP_TRG c_off) {  // zum starten
  unsigned int  ovf_wert=0;

  // benutzter Timer : Timer-1 (16bit)
  // im Mode4 -> ClearTimerOnCompare-A Match

  #if COUNTER_HW_MODE==0
    // Hardware = myAVR MK3-Board 
    TCCR1A=0x00;
    TCCR1B=0x08; // Mode 4
    TCCR1C=0x00;
  #else
    // Hardware = myAVR MK1/MK2-Board + mySmartControl
    TCCR1A=0x00;
    TCCR1B=0x08; // Mode 4
  #endif

  if(c_typ==TYP_US_1000) {
    // counter starten im Mode : Mikrosekunden und Maxwert=1ms
    COUNTER.counter_clk=M0_CLK;
    ovf_wert=M0_OVF;
    COUNTER.counter_faktor=M0_FKT;
  }
  else if(c_typ==TYP_US_10000) {
    // counter starten im Mode : Mikrosekunden und Maxwert=10ms
    COUNTER.counter_clk=M1_CLK;
    ovf_wert=M1_OVF;
    COUNTER.counter_faktor=M1_FKT;
  }
  else if(c_typ==TYP_US_25000) {
    // counter starten im Mode : Mikrosekunden und Maxwert=25ms
    COUNTER.counter_clk=M2_CLK;
    ovf_wert=M2_OVF;
    COUNTER.counter_faktor=M2_FKT;
  }
  else if(c_typ==TYP_MS_1000) {
    // counter starten im Mode : Millisekunden und Maxwert=1sec
    COUNTER.counter_clk=M3_CLK;
    ovf_wert=M3_OVF;
    COUNTER.counter_faktor=M3_FKT;
  }
  else if(c_typ==TYP_MS_4000) {
    // counter starten im Mode : Millisekunden und Maxwert=4sec
    COUNTER.counter_clk=M4_CLK;
    ovf_wert=M4_OVF;
    COUNTER.counter_faktor=M4_FKT;
  }
  else if(c_typ==TYP_S_4) {
    // counter starten im Mode : Sekunden und Maxwert=4sec
    COUNTER.counter_clk=M5_CLK;
    ovf_wert=M5_OVF;
    COUNTER.counter_faktor=M5_FKT;
  }
  else {    
    COUNTER.counter_clk=0; // stop
  }

  // Start und Stop trigger speichern
  COUNTER.start_trg=c_on;
  COUNTER.stop_trg=c_off;  

  // Start-bzw Stop-Trigger mit ICP-Pin
  // Vorsicht !! Reihenfolge ist Wichtig : erst Start dann Stop
  if(c_on==START_LO_ICP) {
    // ICP-Pin auf Eingang und Pull-Up einschalten
    MK3_DDR_ICP&=~MK3_ICP_PIN;
    MK3_PORT_ICP|=MK3_ICP_PIN;
    TCCR1B|=0x80;  // noise canceler enable
    TCCR1B&=~0x40; // trigger auf Lo-Flanke
    // ICP Interrupt aktivieren
    #if COUNTER_HW_MODE==0
      // Hardware = myAVR MK3-Board       
      TIMSK1|=0x20;  // ICP interrupt enable
    #else
      // Hardware = myAVR MK1/MK2-Board + mySmartControl
      TIMSK|=0x20;  // ICP interrupt enable
    #endif
    // ICP AKT-Mode setzen
    COUNTER.akt_mode=ICP_WAIT_START;
  }
  else if(c_on==START_HI_ICP) {
    // ICP-Pin auf Eingang und Pull-Up einschalten
    MK3_DDR_ICP&=~MK3_ICP_PIN;
    MK3_PORT_ICP|=MK3_ICP_PIN;
    TCCR1B|=0x80;  // noise canceler enable
    TCCR1B|=0x40;  // trigger auf Hi-Flanke
    // ICP Interrupt aktivieren
    #if COUNTER_HW_MODE==0
      // Hardware = myAVR MK3-Board       
      TIMSK1|=0x20;  // ICP interrupt enable
    #else
      // Hardware = myAVR MK1/MK2-Board + mySmartControl
      TIMSK|=0x20;  // ICP interrupt enable
    #endif
    // ICP AKT-Mode setzen
    COUNTER.akt_mode=ICP_WAIT_START;
  }
  else if(c_off==STOP_LO_ICP) {
    // ICP-Pin auf Eingang und Pull-Up einschalten
    MK3_DDR_ICP&=~MK3_ICP_PIN;
    MK3_PORT_ICP|=MK3_ICP_PIN;
    TCCR1B|=0x80;  // noise canceler enable
    TCCR1B&=~0x40; // trigger auf Lo-Flanke
    // ICP Interrupt aktivieren
    #if COUNTER_HW_MODE==0
      // Hardware = myAVR MK3-Board       
      TIMSK1|=0x20;  // ICP interrupt enable
    #else
      // Hardware = myAVR MK1/MK2-Board + mySmartControl
      TIMSK|=0x20;  // ICP interrupt enable
    #endif
    // ICP AKT-Mode setzen
    COUNTER.akt_mode=ICP_WAIT_STOP;
  }
  else if(c_off==STOP_HI_ICP) {
    // ICP-Pin auf Eingang und Pull-Up einschalten
    MK3_DDR_ICP&=~MK3_ICP_PIN;
    MK3_PORT_ICP|=MK3_ICP_PIN;
    TCCR1B|=0x80;  // noise canceler enable
    TCCR1B|=0x40;  // trigger auf Hi-Flanke
    // ICP Interrupt aktivieren
    #if COUNTER_HW_MODE==0
      // Hardware = myAVR MK3-Board       
      TIMSK1|=0x20;  // ICP interrupt enable
    #else
      // Hardware = myAVR MK1/MK2-Board + mySmartControl
      TIMSK|=0x20;  // ICP interrupt enable
    #endif
    // ICP AKT-Mode setzen
    COUNTER.akt_mode=ICP_WAIT_STOP;
  }
  else if(c_off==CYCLE_ICP) {
    // ICP-Pin auf Eingang und Pull-Up einschalten
    MK3_DDR_ICP&=~MK3_ICP_PIN;
    MK3_PORT_ICP|=MK3_ICP_PIN;
    TCCR1B|=0x80;  // noise canceler enable
    TCCR1B|=0x40;  // trigger auf Hi-Flanke
    // ICP Interrupt aktivieren
    #if COUNTER_HW_MODE==0
      // Hardware = myAVR MK3-Board       
      TIMSK1|=0x20;  // ICP interrupt enable
    #else
      // Hardware = myAVR MK1/MK2-Board + mySmartControl
      TIMSK|=0x20;  // ICP interrupt enable
    #endif
    // ICP AKT-Mode setzen
    COUNTER.akt_mode=ICP_CYCLE;
  }
  else {
    #if COUNTER_HW_MODE==0
      // Hardware = myAVR MK3-Board       
      TIMSK1&=~0x20;  // ICP interrupt disable
    #else
      // Hardware = myAVR MK1/MK2-Board + mySmartControl
      TIMSK&=~0x20;  // ICP interrupt disable
    #endif
    // ICP AKT-Mode setzen
    COUNTER.akt_mode=ICP_DISABLE;
  }

  // COMP-A Interrupt aktivieren
  #if COUNTER_HW_MODE==0
    // Hardware = myAVR MK3-Board     
    TIMSK1|=0x02;  // Comp-A interrupt enable
  #else
    // Hardware = myAVR MK1/MK2-Board + mySmartControl
    TIMSK|=0x10;  // Comp-A interrupt enable
  #endif

  // Counter Werte init
  TCNT1H=0x00;
  TCNT1L=0x00;
  // OVF-Wert setzen
  OCR1AH=(ovf_wert>>8);
  OCR1AL=(ovf_wert&0x00FF);

  // event zurucksetzen
  COUNTER.counter_event=NO_EVENT;
  COUNTER.counter_wert=0;

  if(c_on==START_NOW) {
    // Counter starten mit Vorteiler
    TCCR1B|=COUNTER.counter_clk;
  }
}

//-----------------------------------------------------------------------------
void MK3_COUNTER_START(void) {                              // zum starten
  // event zurucksetzen
  COUNTER.counter_event=NO_EVENT;
  COUNTER.counter_wert=0;

  // stop Trigger auf ICP
  if(COUNTER.stop_trg!=STOP_FKT) {
    // ICP Interrupt aktivieren
    #if COUNTER_HW_MODE==0
      // Hardware = myAVR MK3-Board       
      TIMSK1|=0x20;  // ICP interrupt enable
    #else
      // Hardware = myAVR MK1/MK2-Board + mySmartControl
      TIMSK|=0x20;  // ICP interrupt enable
    #endif
  }

  // COMP-A Interrupt aktivieren
  #if COUNTER_HW_MODE==0
    // Hardware = myAVR MK3-Board     
    TIMSK1|=0x02;  // Comp-A interrupt enable
  #else
    // Hardware = myAVR MK1/MK2-Board + mySmartControl
    TIMSK|=0x10;  // Comp-A interrupt enable
  #endif

  // Counter starten mit Vorteiler
  TCCR1B|=COUNTER.counter_clk;
}

//-----------------------------------------------------------------------------
void MK3_COUNTER_STOP(void) {                              // zum stoppen
  // timer stoppen
  unsigned int uint;
  unsigned char lo_byte,hi_byte;

  // Counter stoppen
  TCCR1B&=~0x07;

  // interrupts deaktivieren
  #if COUNTER_HW_MODE==0
    // Hardware = myAVR MK3-Board 
    TIMSK1&=~0x02; // Comp-A disable
    TIMSK1&=~0x20; // ICP disable
  #else
    // Hardware = myAVR MK1/MK2-Board + mySmartControl
    TIMSK&=~0x10; // Comp-A disable
    TIMSK&=~0x20; // ICP disable
  #endif

  if(COUNTER.counter_event==NO_EVENT) {
    // wenn noch kein Event

    // TCNT1 auslesen
    lo_byte=TCNT1L;
    hi_byte=TCNT1H;    
    // Counter Werte zuruecksetzen
    TCNT1H=0x00;
    TCNT1L=0x00;

    // Counter Zeit ausrechnen
    uint=(hi_byte<<8)|(lo_byte);
    COUNTER.counter_wert=uint/COUNTER.counter_faktor;    

    // event setzen
    COUNTER.counter_event=STOP_EVENT;
  }
}



//-----------------------------------------------------------------------------
bool MK3_COUNTER_CHECK(void) { // test ob Counter event eingetreten
  // ret_wert : false = counter event noch nicht eingetreten
  //            true  = counter event ist eingetreten

  bool ret_wert=false;

  if(COUNTER.counter_event!=NO_EVENT) ret_wert=true; // wenn event eingetreten

  return(ret_wert);
}

//-----------------------------------------------------------------------------
CNT_EVENT MK3_COUNTER_READ_EVENT(void) {                // Event auslesen
  CNT_EVENT ret_wert=NO_EVENT;

  // counter event zurueckgeben
  ret_wert=COUNTER.counter_event;

  return(ret_wert);
}

//-----------------------------------------------------------------------------
unsigned int MK3_COUNTER_READ_TIME(void) {               // Wert auslesen
  unsigned int ret_wert=0;

  // counter_wert zurueckgeben
  ret_wert=COUNTER.counter_wert; 

  return(ret_wert);
}


//-----------------------------------------------------------------------------
// ISR von Timer-1 Compare-A
//-----------------------------------------------------------------------------
ISR (TIMER1_COMPA_vect) {                          // ISR : Timer1 Comp-A
  // bei Timer1 ueberlauf
  unsigned int uint;
  unsigned char lo_byte,hi_byte;

  if(COUNTER.stop_trg==CYCLE_ICP) {
    // Dauermessung

    // OCR1A auslesen
    lo_byte=OCR1AL;
    hi_byte=OCR1AH;    
    // Counter Werte zuruecksetzen
    TCNT1H=0x00;
    TCNT1L=0x00;

    // Counter Zeit ausrechnen
    uint=(hi_byte<<8)|(lo_byte);
    COUNTER.counter_wert=uint/COUNTER.counter_faktor;

    // event setzen
    COUNTER.counter_event=OVF_EVENT;
  }
  else {
    // Einzelmessung

    // counter stoppen
    TCCR1B&=~0x07;

    // interrupts deaktivieren
    #if COUNTER_HW_MODE==0
      // Hardware = myAVR MK3-Board 
      TIMSK1&=~0x02; // Comp-A disable
      TIMSK1&=~0x20; // ICP disable
    #else
      // Hardware = myAVR MK1/MK2-Board + mySmartControl
      TIMSK&=~0x10; // Comp-A disable
      TIMSK&=~0x20; // ICP disable
    #endif

    if(COUNTER.counter_event==NO_EVENT) {
      // wenn noch kein Event

      // OCR1A auslesen
      lo_byte=OCR1AL;
      hi_byte=OCR1AH;      
      // Counter Werte zuruecksetzen
      TCNT1H=0x00;
      TCNT1L=0x00;

      // Counter Zeit ausrechnen
      uint=(hi_byte<<8)|(lo_byte);
      COUNTER.counter_wert=uint/COUNTER.counter_faktor;

      // event setzen
      COUNTER.counter_event=OVF_EVENT;
    }
  }
}


//-----------------------------------------------------------------------------
// ISR von Timer-1 ICP
//-----------------------------------------------------------------------------
ISR (TIMER1_CAPT_vect) {                          // ISR : Timer1 ICP
  // bei Timer1 ICP event
  unsigned int uint;
  unsigned char lo_byte,hi_byte;

  // Counter Werte zuruecksetzen
  TCNT1H=0x00;
  TCNT1L=0x00;

  if(COUNTER.akt_mode==ICP_CYCLE) {
    // Dauermessung

    // ICR1 auslesen
    lo_byte=ICR1L;
    hi_byte=ICR1H;    

    // Counter Zeit ausrechnen
    uint=(hi_byte<<8)|(lo_byte);
    COUNTER.counter_wert=uint/COUNTER.counter_faktor;

    // event setzen
    COUNTER.counter_event=ICP_EVENT;
  }
  else if(COUNTER.akt_mode==ICP_WAIT_STOP) {
    // stop per ICP-Pin

    // counter stoppen
    TCCR1B&=~0x07;

    // interrupts deaktivieren
    #if COUNTER_HW_MODE==0
      // Hardware = myAVR MK3-Board 
      TIMSK1&=~0x02; // Comp-A disable
      TIMSK1&=~0x20; // ICP disable
    #else
      // Hardware = myAVR MK1/MK2-Board + mySmartControl
      TIMSK&=~0x10; // Comp-A disable
      TIMSK&=~0x20; // ICP disable
    #endif

    if(COUNTER.counter_event==NO_EVENT) {
      // wenn noch kein Event

      // ICR1 auslesen
      lo_byte=ICR1L;
      hi_byte=ICR1H;      

      // Counter Zeit ausrechnen
      uint=(hi_byte<<8)|(lo_byte);
      COUNTER.counter_wert=uint/COUNTER.counter_faktor;

      // event setzen
      COUNTER.counter_event=ICP_EVENT;
    }
  }
  else if(COUNTER.akt_mode==ICP_WAIT_START) {
    // start per ICP-Pin 
	
    // Counter starten mit Vorteiler
    TCCR1B|=COUNTER.counter_clk;	  
	
    // eventuell Stop Trigger setzen
    if(COUNTER.stop_trg==STOP_LO_ICP) {
      TCCR1B&=~0x40;  // trigger auf Lo-Flanke
      COUNTER.akt_mode=ICP_WAIT_STOP;    
    }
    else if(COUNTER.stop_trg==STOP_HI_ICP) {      
      TCCR1B|=0x40;  // trigger auf Hi-Flanke
      COUNTER.akt_mode=ICP_WAIT_STOP;
    }
    else if(COUNTER.stop_trg==STOP_FKT) {
      // interrupts deaktivieren
      #if COUNTER_HW_MODE==0
        // Hardware = myAVR MK3-Board 
        TIMSK1&=~0x20; // ICP disable
      #else
        // Hardware = myAVR MK1/MK2-Board + mySmartControl
        TIMSK&=~0x20; // ICP disable
      #endif
      COUNTER.akt_mode=ICP_DISABLE;
    }
    else {
      // zyklische Messung
      COUNTER.akt_mode=ICP_CYCLE;
    }
  }
}
