//-----------------------------------------------------------------------------
// MK3_2560_PWM
// Version 1.1
// 16.06.2012 / UB
// "www.Mikrocontroller-4U.de"
// mc-4u(@)t-online.de
//-----------------------------------------------------------------------------
// Fuer das myAVR MK3 Board (oder MK2 Board)
// mit ATMega-2560 @ 16MHz (oder ATMega8)
// PWM Funktionen (4 fach 8Bit PWM)
// bzw. 1fach 8Bit bei ATMega8
//-----------------------------------------------------------------------------
// das File muss im Ordner
// "myAVR/WinAVR/AVR/Include/UB_MK3_2560/"
// liegen und mit #include <UB_MK3_2560/MK3_2560_PWM.h>
// eingebunden werden
//-----------------------------------------------------------------------------
// Belegt : @MK3 (ATMega2560)
//            PWM Nr. 1 : Timer-0 und Port-B Bit-7
//            PWM Nr. 2 : Timer-0 und Port-G Bit-5
//            PWM Nr. 3 : Timer-2 und Port-B Bit-4
//            PWM Nr. 4 : Timer-2 und Port-H Bit-6
// Belegt : @MK2 (ATMega8)
//            PWM Nr. 1 : Timer-2 und Port-B Bit-3
//
// Hinweis :
//
// > der DEFINE-Schalter "PWM_HW_MODE"
//   muss zur Hardware passen !!
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Hier koennen Funktionen die nicht benutzt werden
// abgeschaltet werden...einfach den DEFINE von "1" auf "0" aendern
//-----------------------------------------------------------------------------
#define USE_MK3_PWM_1  1
#define USE_MK3_PWM_2  1
#define USE_MK3_PWM_3  1
#define USE_MK3_PWM_4  1



//-----------------------------------------------------------------------------
// Hardware auswahl
//-----------------------------------------------------------------------------
#define PWM_HW_MODE    0       // 0=MK3-Board, 1=MK1/MK2-Board/mySmartControl


//-----------------------------------------------------------------------------
// INFOS zur Benutzung :
// 01. Funktion "MK3_INIT_PWM" starten (fuer jeden PWM-Kanal)
// 02. Funktion "MK3_PWM_SET" zum aendern des PWM-Wertes
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Defines
//-----------------------------------------------------------------------------
#define PWM_VORTEILER 0x02 // 0x02 -> Vorteiler = 8 -> PWM_Frq = ca. 3,9 kHz


//-----------------------------------------------------------------------------
// Enumerationen fuer die Funktioen
//-----------------------------------------------------------------------------
typedef enum {
  PWM_1 = 0,    // PWM Nr. 1 an Port-B7
  PWM_2,        // PWM Nr. 2 an Port-G5
  PWM_3,        // PWM Nr. 3 an Port-B4
  PWM_4         // PWM Nr. 4 an Port-H6
}PWMPORT; 

typedef enum {
  PWM_LO =0,     // Lo_Pegel und Hi_Pulse
  PWM_HI         // Hi_Pegel und Lo_Pulse
}PWMTYP;


//-----------------------------------------------------------------------------
// Funktionen die vom MAIN benutzt werden koennen
//-----------------------------------------------------------------------------
void MK3_INIT_PWM(PWMPORT port,PWMTYP typ);         // zum initialisieren
void MK3_PWM_SET(PWMPORT port, unsigned char wert); // PWM-Wert setzen



//-----------------------------------------------------------------------------
// Port-Pin definition
//-----------------------------------------------------------------------------
#if PWM_HW_MODE==0
  // Hardware = myAVR MK3-Board
  #define MK3_DDR_PWM1  DDRB   // DDR-Register vom PWM-1 Pin
  #define MK3_DDR_PWM2  DDRG   // DDR-Register vom PWM-2 Pin
  #define MK3_DDR_PWM3  DDRB   // DDR-Register vom PWM-3 Pin
  #define MK3_DDR_PWM4  DDRH   // DDR-Register vom PWM-4 Pin
  #define MK3_PORT_PWM1 PORTB  // Port vom PWM-1 Pin
  #define MK3_PORT_PWM2 PORTG  // Port vom PWM-2 Pin
  #define MK3_PORT_PWM3 PORTB  // Port vom PWM-3 Pin
  #define MK3_PORT_PWM4 PORTH  // Port vom PWM-4 Pin
  #define MK3_PWM1_PIN  0x80   // Bit vom PWM-1 Pin
  #define MK3_PWM2_PIN  0x20   // Bit vom PWM-2 Pin
  #define MK3_PWM3_PIN  0x10   // Bit vom PWM-3 Pin
  #define MK3_PWM4_PIN  0x40   // Bit vom PWM-4 Pin
#else
  // Hardware = myAVR MK1/MK2-Board + mySmartControl
  #define MK3_DDR_PWM1  DDRB   // DDR-Register vom PWM-1 Pin
  #define MK3_PORT_PWM1 PORTB  // Port vom PWM-1 Pin
  #define MK3_PWM1_PIN  0x08   // Bit vom PWM-1 Pin
#endif


//-----------------------------------------------------------------------------
void MK3_INIT_PWM(PWMPORT port,PWMTYP typ) { // zum initialisieren


  #if PWM_HW_MODE==0
    // Hardware = myAVR MK3-Board

    // Timer = Mode Nr. 1 = PWM, Phase Correct

    if(port==PWM_1) {
      #if USE_MK3_PWM_1==1
      // Pin auf Ausgang und Lo
      MK3_DDR_PWM1|=MK3_PWM1_PIN;
      MK3_PORT_PWM1&=~MK3_PWM1_PIN;
      // startwerte
      TCNT0=0x00;
      OCR0A=0x00;
      // typ setzen
      if(typ==PWM_LO) {
        TCCR0A|=0x80;
        TCCR0A&=~0x40;
      }
      else {
        TCCR0A|=0x80;
        TCCR0A|=0x40;
      }
      // mode setzen
      TCCR0A|=0x01;
      TCCR0A&=~0x02;
      TCCR0B&=~0x08;
      // Timer starten
      TCCR0B|=PWM_VORTEILER;
      #endif
    }
    else if(port==PWM_2) {
      #if USE_MK3_PWM_2==1
      // Pin auf Ausgang und Lo
      MK3_DDR_PWM2|=MK3_PWM2_PIN;
      MK3_PORT_PWM2&=~MK3_PWM2_PIN;
      // startwerte
      TCNT0=0x00;
      OCR0B=0x00;
      // typ setzen
      if(typ==PWM_LO) {
        TCCR0A|=0x20;
        TCCR0A&=~0x10;
      }
      else {
        TCCR0A|=0x20;
        TCCR0A|=0x10;
      }
      // mode setzen
      TCCR0A|=0x01;
      TCCR0A&=~0x02;
      TCCR0B&=~0x08;
      // Timer starten
      TCCR0B|=PWM_VORTEILER;
      #endif
    }
    else if(port==PWM_3) {
      #if USE_MK3_PWM_3==1
      // Pin auf Ausgang und Lo
      MK3_DDR_PWM3|=MK3_PWM3_PIN;
      MK3_PORT_PWM3&=~MK3_PWM3_PIN;
      // startwerte
      TCNT2=0x00;
      OCR2A=0x00;
      // typ setzen
      if(typ==PWM_LO) {
        TCCR2A|=0x80;
        TCCR2A&=~0x40;
      }
      else {
        TCCR2A|=0x80;
        TCCR2A|=0x40;
      }
      // mode setzen
      TCCR2A|=0x01;
      TCCR2A&=~0x02;
      TCCR2B&=~0x08;
      // Timer starten
      TCCR2B|=PWM_VORTEILER;
      #endif
    }
    else if(port==PWM_4) {
      #if USE_MK3_PWM_4==1
      // Pin auf Ausgang und Lo
      MK3_DDR_PWM4|=MK3_PWM4_PIN;
      MK3_PORT_PWM4&=~MK3_PWM4_PIN;
      // startwerte
      TCNT2=0x00;
      OCR2B=0x00;
      // typ setzen
      if(typ==PWM_LO) {
        TCCR2A|=0x20;
        TCCR2A&=~0x10;
      }
      else {
        TCCR2A|=0x20;
        TCCR2A|=0x10;
      }
      // mode setzen
      TCCR2A|=0x01;
      TCCR2A&=~0x02;
      TCCR2B&=~0x08;
      // Timer starten
      TCCR2B|=PWM_VORTEILER;
      #endif
    }
  #else
    // Hardware = myAVR MK1/MK2-Board + mySmartControl

    // Pin auf Ausgang und Lo
    MK3_DDR_PWM1|=MK3_PWM1_PIN;
    MK3_PORT_PWM1&=~MK3_PWM1_PIN;
    // startwerte
    TCNT2=0x00;
    OCR2=0x00;
    // typ setzen
    if(typ==PWM_LO) {
      TCCR2|=0x20;
      TCCR2&=~0x10;
    }
    else {
      TCCR2|=0x20;
      TCCR2|=0x10;
    }
    // mode setzen
    TCCR2|=0x40;
    TCCR2&=~0x08;    
    // Timer starten
    TCCR2|=PWM_VORTEILER;
  #endif
}

//-----------------------------------------------------------------------------
void MK3_PWM_SET(PWMPORT port, unsigned char wert) { // PWM-Wert setzen
  #if PWM_HW_MODE==0
    // Hardware = myAVR MK3-Board
    if(port==PWM_1) {
      OCR0A=wert;      
    }
    else if(port==PWM_2) {
      OCR0B=wert;
    }
    else if(port==PWM_3) {
      OCR2A=wert;
    }
    else if(port==PWM_4) {
      OCR2B=wert;
    }
  #else
    // Hardware = myAVR MK1/MK2-Board + mySmartControl	
    OCR2=wert;
  #endif	  
}

